# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-11-16 07:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0023_userexamrecord_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='grade',
            name='coin',
            field=models.IntegerField(default=0, verbose_name='\u8003\u7ea7\u6d88\u8017M\u5e01'),
        ),
    ]
