# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-14 06:15
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0003_auto_20180112_1821'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='UserExam',
            new_name='UserExamRecord',
        ),
        migrations.RenameField(
            model_name='userexamrecord',
            old_name='grade_title',
            new_name='grade_name',
        ),
    ]
