# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-25 03:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0018_userexamrecord_cert'),
    ]

    operations = [
        migrations.AddField(
            model_name='userexamrecord',
            name='year',
            field=models.CharField(blank=True, max_length=180, null=True, verbose_name='\u5e74\u5ea6'),
        ),
        migrations.AlterField(
            model_name='gradecertfield',
            name='font_color',
            field=models.CharField(default='#000000', max_length=7, verbose_name='\u5b57\u4f53\u989c\u8272'),
        ),
        migrations.AlterField(
            model_name='gradecertfield',
            name='font_size',
            field=models.IntegerField(default=20, verbose_name='\u5b57\u4f53\u5927\u5c0f'),
        ),
    ]
