# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-19 07:20
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0011_auto_20180118_1719'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='grade',
            name='cert_image',
        ),
    ]
